//      __       __
/*   __/ /_______\ \__     ___ ___ __ _                       _ __ ___ ___
 *__/ / /  .---.  \ \ \___/                                               \___
 *_/ | '  /  / /\  ` | \_/          (C) Copyright 2001, Chris Page         \__
 * \ | |  | / / |  | | / \  Released under the GNU General Public License  /
 *  >| .  \/ /  /  . |<   >--- --- -- -                       - -- --- ---<
 * / \_ \  `/__'  / _/ \ /  This program is free software released under   \
 * \ \__ \_______/ __/ / \   the GNU GPL. Please see the COPYING file in   /
 *  \  \_         _/  /   \   the distribution archive for more details   /
 * //\ \__  ___  __/ /\\ //\                                             /
 *- --\  /_/   \_\  /-- - --\                                           /-----
 *-----\_/       \_/---------\   ___________________________________   /------
 *                            \_/                                   \_/
 *
 * Note to non-developer type persons: this is the source code for texgen, if
 * you're looking for documentation on how to use the program look at the
 * readme.wri file!
 *
 * \author Chris Page 
 * \file texgen.c
 * TexGen creates html files which allow Thief/Thief2 textures to be viewed in
 * a web browser. Unlike Jesse Adams' Thief 2 Texture html document the files
 * generated by this program allow the textures to be viewed at actual size or
 * at a fixed size (which may be bigger than the original texture!) and the 
 * name of the texture is shown in the browser status bar rather than on the 
 * image in a (possibly truncated) cramped form.
 *
 * Contact: Chris <chris@starforge.co.uk> 
 * Updates: http://starforge.co.uk/t2/files/texgen.zip
 *
 * This program is free software released under the GNU General Public
 * License. You may redistribute and/or modify this program under the terms
 * of that license as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * To get a copy of the GNU General Puplic License, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<dirent.h>
#include"htmlsupport.h"

//! Quick list node structure for ceating name lists.
struct NameNode
{
    struct NameNode *prev;
    char *name;
};

char namebuffer[1024]; //!< Save some stack space later (messy, but it's only 1k...)

int familycount  = 0;  //!< Count of the number of family html files generated. 
int texturecount = 0;  //!< Total number of texture files (png or gif) processed. 

FILE *log;

//! Print the command line usage message.
/*1 Print a message that tells the user what arguments this program should be started with 
 */
void usage(char *name)
{
    fprintf(stderr, "Usage: %s [DIRECTORY] [SIZE]\nRecalculate the texture family html tree.\n\n", name);
    fprintf(stderr, "\tDIRECTORY\t Directory containing the family directories\n");
    fprintf(stderr, "\tSIZE\t Width and heigh of texture previews (ommit to view textures at actual size)\n");
    fprintf(stderr, "\t--help\tShow this message.\n\n");
    
    return;
}

//! Sort the specified name list
/*! This is a selection sort which will sort the specified list into ascending order,
 *  don't expect it to be particularly nippy, but since the largest T2 family is
 *  under 100 textures it's not a complete disaster speed wise. Hopefully windows
 *  compilers will understand the strcasecmp!
 */
void sortlist(struct NameNode *list)
{
    struct NameNode *start = list;
    struct NameNode *current;
           char     *temp;

    while(start -> prev) {
        current = start -> prev;
        while(current) {
            if(strcasecmp(current -> name, start -> name) < 0) {
                temp = start -> name;
                start -> name = current -> name;
                current -> name = temp;
            }
            current = current -> prev;
        }
        start = start -> prev;
    }
}
    
//! Create the image display html
/*! This converts the list of image names into a html document called <family>.html
 *  (with <family> replaced by the family name). Each image in the family has an
 *  IMG tag here and the name of the image is displayed in the window status bar
 *  when the mouse is moved over the image.
 */
void writefamilyhtml(char *family, struct NameNode *names, char *minsize)
{
    FILE *html;

    // Calculate the html name and then open it. 
    sprintf(namebuffer, "%s.html", family);

    if((html = fopen(namebuffer, "w")) == NULL) {
        fprintf(log, "Unable to write %s\n", namebuffer);
    } else {
        familycount ++;

        // Header. Note that a stylesheet can be used to change the spacing, colours etc.
        fprintf(html, "<HTML>\n<HEAD>\n<TITLE>%s</TITLE>\n", family);
        fprintf(html, "<link rel=\"stylesheet\" type=\"text/css\" href=\"family.css\" />\n");
        fprintf(html, "<SCRIPT language=\"javascript\">\nfunction showName(name){\ndocument.texture.name.value=name;\nreturn;\n}\n</SCRIPT>\n"); 
        fprintf(html, "</HEAD>\n<BODY>\n<P class=\"title\">Family: %s</P>\n", family);
        fprintf(html, "<P class=\"center\"><FORM NAME=\"texture\"><input NAME=\"name\" TYPE=\"text\" MAXLENGTH=\"60\" SIZE=\"60\"></FORM></P>\n");

        /* Dump out an IMG tag for each texture image. Note that I lop off the extension 
         * for the rollover and alt tag. Images can be forced to a specific size using 
         * minsize. */ 
        while(names) {
            strcpy(namebuffer, names -> name);
            namebuffer[strlen(namebuffer) - 4] = '\0';
            fprintf(html, "<IMG SRC=\"%s/%s\" ALT=\"%s\" ", family, names -> name, namebuffer);
            if(minsize) {
                fprintf(html, "WIDTH=\"%s\" HEIGHT=\"%s\" ", minsize, minsize);
            }
            fprintf(html, "onMouseOver=\"window.status='%s %s'\" onMouseOut=\"window.status=''\" onClick=\"showName('load_a_texture %s %s')\">\n", family, namebuffer, family, namebuffer);
            texturecount ++;
            names = names -> prev;
        }
        
        fprintf(html, "\n</BODY>\n</HEAD>");
        fclose(html);
    }

    return;
}

//! Write the list of families to a html file
/*! Once the family htmls have been created we know which files we need to link to from
 *  a main index. This function creates a html file which is shown in the left frame
 *  of the frameset and from which the user can select thefamily to display.
 */
void writemenu(struct NameNode *names)
{
    FILE *html;

    if((html = fopen("families.html", "w")) == NULL) {
        fprintf(log, "Unable to write %s\n", namebuffer);
    } else {
        familycount ++;

        // Header. Note that a stylesheet can be used to change the spacing, colours etc.
        fprintf(html, "<HTML>\n<HEAD>\n<TITLE>Families</TITLE>\n");
        fprintf(html, "<link rel=\"stylesheet\" type=\"text/css\" href=\"family.css\" />\n");
        fprintf(html, "</HEAD>\n<BODY>\n<P class=\"center\">Family:</P><P class=\"CENTER\">\n");

        /* Dump out an IMG tag for each texture image. Note that I lop off the extension 
         * for the rollover and alt tag. Images can be forced to a specific size using 
         * minsize. */ 
        while(names) {
            fprintf(html, "<A HREF=\"%s.html\" TARGET=\"body\">%s</A><BR>", names -> name, names -> name);
            names = names -> prev;
        }
        
        fprintf(html, "\n</P></BODY>\n</HEAD>");
        fclose(html);
    }

    return;
}

      
//! Attempt to read a family directory
/*! This attempts to open "name" as a directory and read its contents. If name is not
 *  a directory then the open will fail and we exit, otherwise each file with the extension
 *  .png or .gif is added to a list of names which is eventually passed to writefamilyhtml.
 *  Once the html is written the name list is destroyed.
 */
int process_directory(char *name, char *minsize)
{
           DIR      *familyRoot;
    struct NameNode *names  = NULL;
    struct NameNode *newname;
    struct dirent   *texture; 
           int       size   ;

    fprintf(log, "Processing %s... ", name);

    // This is about the only platform-independant way to distinguish between files and dirs..
    if((familyRoot = opendir(name)) == NULL) {
        fprintf(log, "unable to open (not a directory?)\n");
        return 0;
    } else {
        fprintf(log, "opened, scanning for images\n");

        // Each entry in the directory needs to be verified as an image before we remember it,,
        while(texture = readdir(familyRoot)) {
            fprintf(log, "\tProcessing \"%s\"... ", texture -> d_name);
            size = strlen(texture -> d_name);
           
            if(size <= 3) {
                fprintf(log, "not an image?\n");
            } else {
                /* png and gif are probably the best options. I don't know if major
                   browsers recognise pcx.. */
#ifdef PCXALLOWED
                if(strcmp(&texture -> d_name[size - 3], "png") && strcmp(&texture -> d_name[size - 3], "gif") && strcmp(&texture -> d_name[size - 3], "pcx"))  {
#else
                if(strcmp(&texture -> d_name[size - 3], "png") && strcmp(&texture -> d_name[size - 3], "gif")) {
#endif
                    fprintf(log, "not a recognised image type.\n");
                } else {
                    // Add the image name to a simple linked list..
                    if(newname = malloc(sizeof(struct NameNode))) {
                        if(newname -> name = strdup(texture -> d_name)) {
                            newname -> prev = names;
                            names = newname;
                            fprintf(log, "complete.\n");
                        } else {
                            fprintf(log, "error duplicating name.\n");
                        }
                    } else {
                        fprintf(log, " error adding entry.\n");
                    }
                }
            }
        }

        closedir(familyRoot);

        if(names) {
            fprintf(log, "Generating family index\n");
            sortlist(names);
            writefamilyhtml(name, names, minsize);

            // Nuke the name list now that we're done with it..
            do {
                newname = names -> prev;
                free(names -> name);
                free(names);
                names = newname;
            } while(names);
        }
    }

    return 1;
}

int main(int argc, char **argv)
{
           DIR      *textureRoot;
    struct dirent   *family; 
    struct NameNode *families = NULL;
    struct NameNode *newfamily;
           char     *minsize = NULL;

    if(argc < 2) {
        usage(argv[0]);
    } else {
        if(!strcmp(argv[1], "--help")) {
            usage(argv[0]);
        } else {

            if((log = fopen("texgen.log", "w")) == NULL) {
                fprintf(stderr, "Unable to open log file\n");
                exit(1);
            }

            // Has the user specified a minimum size?
            if(argc == 3) {
                // letting through junk here would be a Bad Thing..
                if(atol(argv[2])) {
                    minsize = argv[2];
                }
            }

            if((textureRoot = opendir(argv[1])) == NULL) {
                fprintf(stderr, "Unable to open %s\n", argv[1]);
                fprintf(log, "Unable to open %s\n", argv[1]);
                fclose(log);
                exit(1);
            } else {
                // We need ot be in the same directory as the texture families to work..
                chdir(argv[1]);

                while(family = readdir(textureRoot)) {
                    // We don't want ot process anything starting . (including .. and .)
                    if(family -> d_name[0] != '.') { 

                        // process_directory returns non-zero on success.
                        if(process_directory(family -> d_name, minsize)) {
                            fprintf(log, "Adding family menu entry...\n");
                            if(newfamily = malloc(sizeof(struct NameNode))) {
                                if(newfamily -> name = strdup(family -> d_name)) {
                                    newfamily -> prev = families;
                                    families = newfamily;
                                    fprintf(log, "complete.\n");
                                } else {
                                    fprintf(log, "error duplicating name.\n");
                                }
                            } else {
                                fprintf(log, " error adding entry.\n");
                            }
                        }
                    }
                }

                closedir(textureRoot);

                // write the menu and other top level files and then free the family list.
                if(families) {
                    sortlist(families);
                    writemenu(families);
                    writeframeset();
                    writenotes();

                    do {
                        newfamily = families -> prev;
                        free(families -> name);
                        free(families);
                        families = newfamily;
                    } while(families);
                }
            }
            fclose(log);
            printf("Processed %d textures in %d families\n", texturecount, familycount);
        }
    }

    exit(0);
}
            
